@@ -8,7 +8,6 @@ const Main = imports.ui.main;
 const DND = imports.ui.dnd;
 const Clutter = imports.gi.Clutter;
 const AppletManager = imports.ui.appletManager;
-const Gtk = imports.gi.Gtk;
 const Util = imports.misc.util;
 const Pango = imports.gi.Pango;
 const Mainloop = imports.mainloop;
@@ -16,15 +15,9 @@ const Flashspot = imports.ui.flashspot;
 const ModalDialog = imports.ui.modalDialog;
 const Signals = imports.signals;
 const Gettext = imports.gettext;
+const Cinnamon = imports.gi.Cinnamon;
 
-const COLOR_ICON_HEIGHT_FACTOR = .875;  // Panel height factor for normal color icons
-const PANEL_FONT_DEFAULT_HEIGHT = 11.5; // px
-const PANEL_SYMBOLIC_ICON_DEFAULT_HEIGHT = 1.14 * PANEL_FONT_DEFAULT_HEIGHT; // ems conversion
-const DEFAULT_PANEL_HEIGHT = 25;
-const DEFAULT_ICON_HEIGHT = 22;
-const FALLBACK_ICON_HEIGHT = 22;
-
-const AllowedLayout = {  // the panel layout that an applet is suitable for
+var AllowedLayout = {  // the panel layout that an applet is suitable for
     VERTICAL: 'vertical',
     HORIZONTAL: 'horizontal',
     BOTH: 'both'
@@ -34,15 +27,9 @@ const AllowedLayout = {  // the panel layout that an applet is suitable for
  * #MenuItem
  * @short_description: Deprecated. Use #PopupMenu.PopupIconMenuItem instead.
  */
-function MenuItem(label, icon, callback) {
-    this._init(label, icon, callback);
-}
-
-MenuItem.prototype = {
-    __proto__ : PopupMenu.PopupIconMenuItem.prototype,
-
-    _init: function(label, icon, callback) {
-        PopupMenu.PopupIconMenuItem.prototype._init.call(this, label, icon, St.IconType.SYMBOLIC);
+var MenuItem = class MenuItem extends PopupMenu.PopupIconMenuItem {
+    _init(label, icon, callback) {
+        super._init(label, icon, St.IconType.SYMBOLIC);
         this.connect('activate', callback);
     }
 }
@@ -55,34 +42,21 @@ MenuItem.prototype = {
  *
  * Inherits: PopupMenu.PopupMenu
  */
-function AppletContextMenu(launcher, orientation) {
-    this._init(launcher, orientation);
-}
-
-AppletContextMenu.prototype = {
-    __proto__: PopupMenu.PopupMenu.prototype,
-
-    /**
-     * _init:
-     * @launcher (Applet.Applet): The applet that contains the context menu
-     * @orientation (St.Side): The orientation of the applet
-     *
-     * Constructor function
-     */
-    _init: function(launcher, orientation) {
-        PopupMenu.PopupMenu.prototype._init.call(this, launcher.actor, orientation);
+var AppletContextMenu = class AppletContextMenu extends PopupMenu.PopupMenu {
+    _init(launcher, orientation) {
+        super._init(launcher.actor, orientation);
         Main.uiGroup.add_actor(this.actor);
         this.actor.hide();
         this.connect("open-state-changed", Lang.bind(this, this._onOpenStateChanged, launcher.actor));
         launcher.connect("orientation-changed", Lang.bind(this, function(a, orientation) {
             this.setArrowSide(orientation);
         }));
-    },
+    }
 
-    _onOpenStateChanged: function(menu, open, sourceActor) {
+    _onOpenStateChanged(menu, open, sourceActor) {
         sourceActor.change_style_pseudo_class("checked", open);
     }
-};
+}
 
 /**
  * #AppletPopupMenu:
@@ -92,12 +66,7 @@ AppletContextMenu.prototype = {
  *
  * Inherits: PopupMenu.PopupMenu
  */
-function AppletPopupMenu(launcher, orientation) {
-    this._init(launcher, orientation);
-}
-
-AppletPopupMenu.prototype = {
-    __proto__: PopupMenu.PopupMenu.prototype,
+var AppletPopupMenu = class AppletPopupMenu extends PopupMenu.PopupMenu {
 
     /**
      * _init:
@@ -106,8 +75,8 @@ AppletPopupMenu.prototype = {
      *
      * Constructor function
      */
-    _init: function(launcher, orientation) {
-        PopupMenu.PopupMenu.prototype._init.call(this, launcher.actor, orientation);
+    _init(launcher, orientation) {
+        super._init(launcher.actor, orientation);
         Main.uiGroup.add_actor(this.actor);
         this.actor.hide();
         this.launcher = launcher;
@@ -117,17 +86,17 @@ AppletPopupMenu.prototype = {
         } else if (launcher._applet) {
             launcher._applet.connect("orientation-changed", Lang.bind(this, this._onOrientationChanged));
         }
-    },
+    }
 
-    _onOrientationChanged: function(a, orientation) {
+    _onOrientationChanged(a, orientation) {
         this.setArrowSide(orientation);
-    },
+    }
 
-    _onOpenStateChanged: function(menu, open, sourceActor) {
+    _onOpenStateChanged(menu, open, sourceActor) {
         if (!sourceActor._applet_context_menu.isOpen)
             sourceActor.actor.change_style_pseudo_class("checked", open);
     }
-};
+}
 
 /**
  * #Applet
@@ -146,7 +115,6 @@ AppletPopupMenu.prototype = {
  * @_order (int): The order of the applet within a panel location This is set
  * by appletManager *after* the applet is loaded.
  * @_draggable (Dnd._Draggable): The draggable object of the applet
- * @_scaleMode (boolean): Whether the applet scales according to the panel size
  * @_applet_tooltip (Tooltips.PanelItemTooltip): The tooltip of the applet
  * @_menuManager (PopupMenu.PopupMenuManager): The menu manager of the applet
  * @_applet_context_menu (Applet.AppletContextMenu): The context menu of the applet
@@ -157,11 +125,7 @@ AppletPopupMenu.prototype = {
  *
  * Base applet class that other applets can inherit
  */
-function Applet(orientation, panelHeight, instance_id) {
-    this._init(orientation, panelHeight, instance_id);
-}
-
-Applet.prototype = {
+var Applet = class Applet {
 
     /**
      * _init:
@@ -169,7 +133,12 @@ Applet.prototype = {
      * @panelHeight (int): height of the panel containing the applet
      * @instance_id (int): instance id of the applet
      */
-    _init: function(orientation, panel_height, instance_id) {
+
+    constructor() {
+        return this._init.apply(this, arguments);
+    }
+
+    _init(orientation, panel_height, instance_id) {
 
         this.actor = new St.BoxLayout({ style_class: 'applet-box',
                                         reactive: true,
@@ -188,16 +157,20 @@ Applet.prototype = {
         this.actor._applet = this;  // Backlink to get the applet from its actor
                                     // (handy when we want to know stuff about a particular applet within the panel)
         this.actor._delegate = this;
+
+        this.panel = null;
         this._order = 0;        // Defined in gsettings, this is the order of the applet within a panel location.
                                 // This value is set by Cinnamon when loading/listening_to gsettings.
         this._newOrder = null;      //  Used when moving an applet
         this._panelLocation = null;     // Backlink to the panel location our applet is in, set by Cinnamon.
+        this.locationLabel = 'right';
         this._newPanelId = null;  //  Used when moving an applet
         this._newPanelLocation = null;  //  Used when moving an applet
         this._applet_enabled = true;    // Whether the applet is enabled or not (if not it hides in the panel as if it wasn't there)
         this._orientation = orientation;  // orientation of the panel the applet is on  St.Side.TOP BOTTOM LEFT RIGHT
+        this._lastIconType = St.IconType.FULLCOLOR;
+        this._iconSize = null;
 
-        this._panelHeight = panel_height ? panel_height : 25;
         this.instance_id = instance_id; // Needed by appletSettings
         this._uuid = null;      // Defined in gsettings, set by Cinnamon.
         this._hook = null;      // Defined in metadata.json, set by appletManager
@@ -208,21 +181,6 @@ Applet.prototype = {
         this._draggable.connect('drag-cancelled', Lang.bind(this, this._onDragCancelled));
         this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
 
-        try {
-            if (AppletManager.enabledAppletDefinitions.idMap[instance_id]) {
-                this._scaleMode = AppletManager.enabledAppletDefinitions.idMap[instance_id].panel.scaleMode;
-            } else {
-                throw new Error();
-            }
-        } catch (e) {
-            // Sometimes applets are naughty and don't pass us our instance_id. In that case, we just find the first non-empty panel and pretend we are on it.
-            for (let i in Main.panelManager.panels) {
-                this._scaleMode = true;
-                if (Main.panelManager.panels[i]) {
-                    this._scaleMode = Main.panelManager.panels[i].scaleMode;
-                }
-            }
-        }
         this._applet_tooltip_text = "";
 
         this.context_menu_item_remove = null;
@@ -232,66 +190,102 @@ Applet.prototype = {
         this._panelEditModeChangedId = global.settings.connect('changed::panel-edit-mode', Lang.bind(this, function() {
             this._setAppletReactivity();
         }));
-    },
+
+        // FIXME: Cinnamon should be providing a sandbox environment for extensions, and not depend on data passed
+        // from the extension for basic state that we are already keeping track of in appletManager. Since applets
+        // need icon sizes available immediately in their constructor, this has to stay for now.
+        if (instance_id) {
+            this._getPanelInfo(instance_id);
+        } else {
+            setTimeout(() => this._getPanelInfo(), 0);
+        }
+    }
+
+    _getPanelInfo(instance_id) {
+        if (!instance_id) instance_id = this.instance_id;
+        let appletDefinition = AppletManager.getAppletDefinition({applet_id: instance_id});
+        if (appletDefinition) {
+            let panelIndex = Util.findIndex(Main.panelManager.panels, function(panel) {
+                return panel && (panel.panelId === appletDefinition.panelId);
+            });
+            if (panelIndex > -1) {
+                let panel = Main.panelManager.panels[panelIndex];
+                this.locationLabel = appletDefinition.location_label;
+                this.panel = panel;
+                this._uuid = appletDefinition.uuid;
+            } else {
+                global.logWarning(`[Applet] No panel found for ${instance_id}`);
+            }
+        } else {
+            throw new Error(`[Applet] Unable to find definition for applet ${instance_id}`);
+        }
+    }
 
     /* FIXME:  This makes no sense - inhibit flag should = panel edit mode, right?
      *         Needs fixed in dnd.js also, it expects this backwards logic right now
      */
 
-    _setAppletReactivity: function() {
+    _setAppletReactivity() {
         this._draggable.inhibit = !global.settings.get_boolean('panel-edit-mode');
-    },
+    }
 
-    _onDragBegin: function() {
+    _onDragBegin() {
         this._dragging = true;
         this._applet_tooltip.hide();
         this._applet_tooltip.preventShow = true;
         Main.panelManager.resetPanelDND();
-    },
+    }
 
-    _onDragEnd: function() {
+    _onDragEnd() {
         this._dragging = false;
         this._applet_tooltip.preventShow = false;
-    },
+    }
 
-    _onDragCancelled: function() {
+    _onDragCancelled() {
         this._dragging = false;
         this._applet_tooltip.preventShow = false;
-    },
+    }
 
-    getDragActor: function() {
+    getDragActor() {
         let clone = new Clutter.Clone({ source: this.actor });
         clone.width = this.actor.width;
         clone.height = this.actor.height;
         return clone;
-    },
+    }
 
     // Returns the original actor that should align with the actor
     // we show as the item is being dragged.
-    getDragActorSource: function() {
+    getDragActorSource() {
         return this.actor;
-    },
+    }
 
-    _onButtonPressEvent: function (actor, event) {
-        if (this._applet_enabled) {
-            if (event.get_button() == 1) {
-                if (!this._draggable.inhibit) {
-                    return false;
-                } else {
-                    if (this._applet_context_menu.isOpen) {
-                        this._applet_context_menu.toggle();
-                    }
-                    this.on_applet_clicked(event);
-                }
-            }
-            if (event.get_button() == 3) {
-                if (this._applet_context_menu._getMenuItems().length > 0) {
+    _onButtonPressEvent (actor, event) {
+        if (!this._applet_enabled) {
+            return false;
+        }
+
+        let button = event.get_button();
+        if (button < 3) {
+            if (!this._draggable.inhibit) {
+                return false;
+            } else {
+                if (this._applet_context_menu.isOpen) {
                     this._applet_context_menu.toggle();
                 }
             }
         }
+
+        if (button === 1) {
+            this.on_applet_clicked(event);
+        } else if (button === 2) {
+            this.on_applet_middle_clicked(event);
+        } else if (button === 3) {
+            if (this._applet_context_menu._getMenuItems().length > 0) {
+                this._applet_context_menu.toggle();
+            }
+        }
         return true;
-    },
+    }
 
     /**
      * set_applet_tooltip:
@@ -299,12 +293,15 @@ Applet.prototype = {
      *
      * Sets the tooltip of the applet
      */
-    set_applet_tooltip: function (text) {
+    set_applet_tooltip (text) {
         if (text != this._applet_tooltip_text) {
             this._applet_tooltip_text = text;
             this._applet_tooltip.set_text(text);
         }
-    },
+        if (text === "") {
+            this._applet_tooltip.hide();
+        }
+    }
 
     /**
      * set_applet_enabled:
@@ -313,12 +310,12 @@ Applet.prototype = {
      * Sets whether the applet is enabled or not. A disabled applet sets its
      * padding to 0px and doesn't react to clicks
      */
-    set_applet_enabled: function (enabled) {
+    set_applet_enabled (enabled) {
         if (enabled != this._applet_enabled) {
             this._applet_enabled = enabled;
             this.actor.visible = enabled;
         }
-    },
+    }
 
     /**
      * on_applet_clicked:
@@ -328,25 +325,41 @@ Applet.prototype = {
      *
      * This is meant to be overridden in individual applets.
      */
-    on_applet_clicked: function(event) {
+    on_applet_clicked(event) {
         // Implemented by Applets
-    },
+    }
+
+    /**
+     * on_applet_middle_clicked:
+     * @event (Clutter.Event): the event object
+     *
+     * This function is called when the applet is clicked with the middle mouse button.
+     *
+     * This is meant to be overridden in individual applets.
+     */
+    on_applet_middle_clicked(event) {
+        // Implemented by Applets
+    }
 
 
     /**
      * on_applet_instances_changed:
+     * @instance (Applet) the instance that was changed
      *
      * This function is called when an applet *of the same uuid* is added or
      * removed from the panels. It is intended to assist in delegation of
      * responsibilities between duplicate applet instances.
      *
+     * Applets should not create any references to @instance, since that
+     * could impede garbage collection.
+     *
      * This is meant to be overridden in individual applets
      */
-    on_applet_instances_changed: function() {
+    on_applet_instances_changed() {
 
-    },
+    }
 
-    on_applet_added_to_panel_internal: function(userEnabled) {
+    on_applet_added_to_panel_internal(userEnabled) {
         if (userEnabled) {
             Mainloop.timeout_add(300, Lang.bind(this, function() {
                 let [x, y] = this.actor.get_transformed_position();
@@ -357,10 +370,12 @@ Applet.prototype = {
             }));
         }
 
+        this._panelSizeChangeId = this.panel.connect('size-changed', () => this.on_panel_height_changed_internal());
+        this._panelIconSizeChangeId = this.panel.connect('icon-size-changed', () => this.on_panel_icon_size_changed_internal());
         this.on_applet_added_to_panel(userEnabled);
 
-        Main.AppletManager.callAppletInstancesChanged(this._uuid);
-    },
+        Main.AppletManager.callAppletInstancesChanged(this._uuid, this);
+    }
 
     /**
      * on_applet_added_to_panel:
@@ -369,8 +384,8 @@ Applet.prototype = {
      *
      * This is meant to be overridden in individual applets.
      */
-    on_applet_added_to_panel: function(userEnabled) {
-    },
+    on_applet_added_to_panel(userEnabled) {
+    }
 
     /**
      * on_applet_removed_from_panel:
@@ -379,16 +394,28 @@ Applet.prototype = {
      *
      * This is meant to be overridden in individual applets.
      */
-    on_applet_removed_from_panel: function(deleteConfig) {
-    },
+    on_applet_removed_from_panel(deleteConfig) {
+    }
+
+    /**
+     * on_applet_reloaded:
+     *
+     * This function is called by appletManager when the applet is reloaded.
+     *
+     * This is meant to be overridden in individual applets.
+     */
+    on_applet_reloaded(deleteConfig) {
+    }
 
     // should only be called by appletManager
-    _onAppletRemovedFromPanel: function(deleteConfig) {
+    _onAppletRemovedFromPanel(deleteConfig) {
         global.settings.disconnect(this._panelEditModeChangedId);
+        this.panel.disconnect(this._panelSizeChangeId);
+        this.panel.disconnect(this._panelIconSizeChangeId);
         this.on_applet_removed_from_panel(deleteConfig);
 
-        Main.AppletManager.callAppletInstancesChanged(this._uuid);
-    },
+        Main.AppletManager.callAppletInstancesChanged(this._uuid, this);
+    }
 
     /**
      * setOrientationInternal:
@@ -397,7 +424,7 @@ Applet.prototype = {
      * Sets the orientation of the St.BoxLayout.
      *
      */
-    setOrientationInternal: function (orientation) {
+    setOrientationInternal (orientation) {
         if (orientation == St.Side.LEFT || orientation == St.Side.RIGHT) {
             this.actor.add_style_class_name('vertical');
             this.actor.set_important(true);
@@ -407,7 +434,7 @@ Applet.prototype = {
             this.actor.remove_style_class_name('vertical');
             this.actor.set_vertical(false);
         }
-    },
+    }
 
     /**
      * setOrientation:
@@ -417,12 +444,22 @@ Applet.prototype = {
      *
      * This function should only be called by appletManager
      */
-    setOrientation: function (orientation) {
+    setOrientation (orientation) {
+        this._orientation = orientation;
         this.setOrientationInternal(orientation);
         this.on_orientation_changed(orientation);
         this.emit("orientation-changed", orientation);
         this.finalizeContextMenu();
-    },
+
+        if (typeof this.set_applet_label === 'function' && this._applet_label instanceof St.Label) {
+            this.set_applet_label(this._applet_label.get_text());
+        }
+
+        // FIXME: This function will be called from AppletManager before the panel is
+        // assigned. When the panel orientation changes, it becomes unavailable,
+        // so we need to check for this.panel here.
+        if (this.panel) this.on_panel_icon_size_changed_internal();
+    }
 
     /**
      * setAllowedLayout:
@@ -432,9 +469,9 @@ Applet.prototype = {
      * AllowedLayout.HORIZONTAL, AllowedLayout.VERTICAL, and
      * AllowedLayout.BOTH.
      */
-    setAllowedLayout: function (layout) {
+    setAllowedLayout (layout) {
         this._allowedLayout = layout;
-    },
+    }
 
     /**
      * getAllowedLayout:
@@ -443,9 +480,9 @@ Applet.prototype = {
      *
      * Returns (Applet.AllowedLayout): The allowed layout of the applet
      */
-    getAllowedLayout: function() {
+    getAllowedLayout() {
         return this._allowedLayout;
-    },
+    }
 
     /**
      * on_orientation_changed:
@@ -455,32 +492,28 @@ Applet.prototype = {
      *
      * This is meant to be overridden in individual applets.
      */
-    on_orientation_changed: function(orientation) {
+    on_orientation_changed(orientation) {
         // Implemented by Applets
-    },
+    }
 
-    /**
-     * setPanelHeight:
-     * @panelHeight (int): panelHeight
-     *
-     * Sets the panel height property of the applet.
-     */
-    setPanelHeight: function (panel_height) {
-        if (panel_height && panel_height > 0) {
-            this._panelHeight = panel_height;
-        }
-        this._scaleMode = AppletManager.enabledAppletDefinitions.idMap[this.instance_id].panel.scaleMode;
-        this.on_panel_height_changed_internal();
-    },
+    getPanelIconSize(iconType = St.IconType.FULLCOLOR) {
+        // If no panel, then the panel probably was added with pre-existing applet
+        // definitions associated to it. This means there's no zone config, so return early.
+        if (!this.panel) return;
+
+        this._lastIconType = iconType;
+        this._iconSize = this.panel.getPanelZoneIconSize(this.locationLabel, iconType);
+        return this._iconSize;
+    }
 
     /**
      * on_panel_height_changed_internal:
      *
      * This function is called when the panel containing the applet changes height
      */
-    on_panel_height_changed_internal: function() {
+    on_panel_height_changed_internal() {
         this.on_panel_height_changed();
-    },
+    }
 
     /**
      * on_panel_height_changed:
@@ -489,11 +522,49 @@ Applet.prototype = {
      *
      * This is meant to be overridden in individual applets.
      */
-    on_panel_height_changed: function() {
+    on_panel_height_changed() {
         // Implemented byApplets
-    },
+    }
 
-    finalizeContextMenu: function () {
+    on_panel_icon_size_changed_internal() {
+        let size = this.panel.getPanelZoneIconSize(this.locationLabel, this._lastIconType);
+        if (!this._iconSize || this._iconSize !== size) {
+            this._iconSize = size;
+
+            if (this._applet_icon) {
+                this._applet_icon.set_icon_size(size);
+            }
+
+            this.on_panel_icon_size_changed(size);
+        }
+    }
+
+    /**
+     * on_panel_icon_size_changed:
+     * @size (number): new icon size
+     *
+     * This function is called when the icon size preference for the panel zone
+     * containing this applet is changed.
+     *
+     * This is meant to be overridden in individual applets.
+     */
+    on_panel_icon_size_changed() {
+        // Implemented byApplets
+    }
+
+    confirmRemoveApplet (event) {
+        if (Clutter.ModifierType.CONTROL_MASK & Cinnamon.get_event_state(event)) {
+            AppletManager._removeAppletFromPanel(this._uuid, this.instance_id);
+        } else {
+            let dialog = new ModalDialog.ConfirmDialog(
+                _("Are you sure you want to remove %s?").format(this._meta.name),
+                () => AppletManager._removeAppletFromPanel(this._uuid, this.instance_id)
+            );
+            dialog.open();
+        }
+    }
+
+    finalizeContextMenu () {
 
         // Add default context menus if we're in panel edit mode, ensure their removal if we're not
         let items = this._applet_context_menu._getMenuItems();
@@ -503,9 +574,7 @@ Applet.prototype = {
                 .format(this._(this._meta.name)),
                    "edit-delete",
                    St.IconType.SYMBOLIC);
-            this.context_menu_item_remove.connect('activate', Lang.bind(this, function() {
-                AppletManager._removeAppletFromPanel(this._uuid, this.instance_id);
-            }));
+            this.context_menu_item_remove.connect('activate', (actor, event) => this.confirmRemoveApplet(event));
         }
 
         if (this.context_menu_item_about == null) {
@@ -539,10 +608,10 @@ Applet.prototype = {
         if (items.indexOf(this.context_menu_item_remove) == -1) {
             this._applet_context_menu.addMenuItem(this.context_menu_item_remove);
         }
-    },
+    }
 
     // translation
-    _: function(str) {
+    _(str) {
         // look into the text domain first
         let translated = Gettext.dgettext(this._uuid, str);
 
@@ -551,7 +620,7 @@ Applet.prototype = {
             return translated;
         // else, use the default cinnamon domain
         return _(str);
-    },
+    }
 
     /**
      * highlight:
@@ -559,17 +628,26 @@ Applet.prototype = {
      *
      * Turns on/off the highlight of the applet
      */
-    highlight: function(highlight) {
+    highlight(highlight) {
         this.actor.change_style_pseudo_class("highlight", highlight);
-    },
+    }
 
-    openAbout: function() {
-        new ModalDialog.SpicesAboutDialog(this._meta, "applets");
-    },
+    openAbout() {
+        Util.spawnCommandLine("xlet-about-dialog applets " + this._uuid);
+    }
 
-    configureApplet: function() {
+    configureApplet() {
         Util.spawnCommandLine("xlet-settings applet " + this._uuid + " " + this.instance_id);
     }
+
+    get _panelHeight() {
+        return this.panel.height;
+    }
+
+    get _scaleMode() {
+        global.logWarning(`[Applet/${this._uuid}] Use of scaleMode is deprecated.`);
+        return true;
+    }
 };
 Signals.addSignalMethods(Applet.prototype);
 
@@ -583,12 +661,7 @@ Signals.addSignalMethods(Applet.prototype);
  *
  * Inherits: Applet.Applet
  */
-function IconApplet(orientation, panel_height, instance_id) {
-    this._init(orientation, panel_height, instance_id);
-}
-
-IconApplet.prototype = {
-    __proto__: Applet.prototype,
+var IconApplet = class IconApplet extends Applet {
 
     /**
      * _init:
@@ -596,15 +669,15 @@ IconApplet.prototype = {
      * @panelHeight (int): height of the panel containing the applet
      * @instance_id (int): instance id of the applet
      */
-    _init: function(orientation, panel_height, instance_id) {
-        Applet.prototype._init.call(this, orientation, panel_height, instance_id);
+    _init(orientation, panel_height, instance_id) {
+        super._init(orientation, panel_height, instance_id);
 
         this._applet_icon_box = new St.Bin(); // https://developer.gnome.org/st/stable/StBin.htm
 
         this._applet_icon_box.set_fill(true,true);
         this._applet_icon_box.set_alignment(St.Align.MIDDLE,St.Align.MIDDLE);
         this.actor.add(this._applet_icon_box);
-    },
+    }
 
     /**
      * set_applet_icon_name:
@@ -614,13 +687,13 @@ IconApplet.prototype = {
      *
      * The icon will be full color
      */
-    set_applet_icon_name: function (icon_name) {
+    set_applet_icon_name (icon_name) {
         this._ensureIcon();
 
         this._applet_icon.set_icon_name(icon_name);
         this._applet_icon.set_icon_type(St.IconType.FULLCOLOR);
         this._setStyle();
-    },
+    }
 
     /**
      * set_applet_icon_symbolic_name:
@@ -630,13 +703,13 @@ IconApplet.prototype = {
      *
      * The icon will be symbolic
      */
-    set_applet_icon_symbolic_name: function (icon_name) {
+    set_applet_icon_symbolic_name (icon_name) {
         this._ensureIcon();
 
         this._applet_icon.set_icon_name(icon_name);
         this._applet_icon.set_icon_type(St.IconType.SYMBOLIC);
         this._setStyle();
-    },
+    }
 
     /**
      * set_applet_icon_path:
@@ -646,7 +719,7 @@ IconApplet.prototype = {
      *
      * The icon will be full color
      */
-    set_applet_icon_path: function (icon_path) {
+    set_applet_icon_path (icon_path) {
         this._ensureIcon();
 
         try {
@@ -657,7 +730,7 @@ IconApplet.prototype = {
         } catch (e) {
             global.log(e);
         }
-    },
+    }
 
     /**
      * set_applet_icon_symbolic_path:
@@ -667,7 +740,7 @@ IconApplet.prototype = {
      *
      * The icon will be symbolic
      */
-    set_applet_icon_symbolic_path: function(icon_path) {
+    set_applet_icon_symbolic_path(icon_path) {
         this._ensureIcon();
 
         try {
@@ -678,48 +751,33 @@ IconApplet.prototype = {
         } catch (e) {
             global.log(e);
         }
-    },
+    }
 
-    _ensureIcon: function() {
+    _ensureIcon() {
         if (!this._applet_icon || !(this._applet_icon instanceof St.Icon))
             this._applet_icon = new St.Icon({ reactive: true, track_hover: true, style_class: 'applet-icon'});
 
         this._applet_icon_box.set_child(this._applet_icon);
-    },
+    }
 
-    _setStyle: function() {
-
-        let symb_scaleup = ((this._panelHeight / DEFAULT_PANEL_HEIGHT) * PANEL_SYMBOLIC_ICON_DEFAULT_HEIGHT) / global.ui_scale;
-        let fullcolor_scaleup = this._panelHeight * COLOR_ICON_HEIGHT_FACTOR / global.ui_scale;
+    _setStyle() {
         let icon_type = this._applet_icon.get_icon_type();
 
-        switch (icon_type) {
-            case St.IconType.FULLCOLOR:
-                this._applet_icon.set_icon_size(this._scaleMode ?
-                                                fullcolor_scaleup :
-                                                DEFAULT_ICON_HEIGHT);
-                this._applet_icon.set_style_class_name('applet-icon');
-            break;
-            case St.IconType.SYMBOLIC:
-                this._applet_icon.set_icon_size(this._scaleMode ?
-                                                symb_scaleup :
-                                                -1);
-                this._applet_icon.set_style_class_name('system-status-icon');
-            break;
-            default:
-                this._applet_icon.set_icon_size(this._scaleMode ?
-                                                symb_scaleup :
-                                                -1);
-                                                this._applet_icon.set_style_class_name('system-status-icon');
+        if (icon_type === St.IconType.FULLCOLOR) {
+            this._applet_icon.set_icon_size(this.getPanelIconSize(St.IconType.FULLCOLOR));
+            this._applet_icon.set_style_class_name('applet-icon');
+        } else {
+            this._applet_icon.set_icon_size(this.getPanelIconSize(St.IconType.SYMBOLIC));
+            this._applet_icon.set_style_class_name('system-status-icon');
         }
-    },
+    }
 
-    on_panel_height_changed_internal: function() {
+    on_panel_height_changed_internal() {
         if (this._applet_icon)
             this._setStyle();
         this.on_panel_height_changed();
     }
-};
+}
 
 /**
  * #TextApplet:
@@ -730,12 +788,7 @@ IconApplet.prototype = {
  *
  * Inherits: Applet.Applet
  */
-function TextApplet(orientation, panel_height, instance_id) {
-    this._init(orientation, panel_height, instance_id);
-}
-
-TextApplet.prototype = {
-    __proto__: Applet.prototype,
+var TextApplet = class TextApplet extends Applet {
 
     /**
      * _init:
@@ -746,8 +799,8 @@ TextApplet.prototype = {
      * Note that suitability for display in a vertical panel is handled by having applets declare
      * they work OK, handled elsewhere
      */
-    _init: function(orientation, panel_height, instance_id) {
-        Applet.prototype._init.call(this, orientation, panel_height, instance_id);
+    _init(orientation, panel_height, instance_id) {
+        super._init(orientation, panel_height, instance_id);
         this._applet_label = new St.Label({ reactive: true,
                                             track_hover: true,
                                             style_class: 'applet-label'});
@@ -759,7 +812,7 @@ TextApplet.prototype = {
         this.actor.add(this._layoutBin, { y_align: St.Align.MIDDLE,
                                           y_fill: false });
         this.actor.set_label_actor(this._applet_label);
-    },
+    }
 
     /**
      * set_applet_label:
@@ -767,13 +820,13 @@ TextApplet.prototype = {
      *
      * Sets the text of the actor to @text
      */
-    set_applet_label: function (text) {
+    set_applet_label (text) {
         this._applet_label.set_text(text);
-    },
-
-    on_applet_added_to_panel: function() {
     }
-};
+
+    on_applet_added_to_panel() {
+    }
+}
 
 /**
  * #TextIconApplet:
@@ -786,12 +839,7 @@ TextApplet.prototype = {
  * Note that suitability for display in a vertical panel is handled by having applets declare
  * they work OK, handled elsewhere
  */
-function TextIconApplet(orientation, panel_height, instance_id) {
-    this._init(orientation, panel_height, instance_id);
-}
-
-TextIconApplet.prototype = {
-    __proto__: IconApplet.prototype,
+var TextIconApplet = class TextIconApplet extends IconApplet {
 
     /**
      * _init:
@@ -799,8 +847,8 @@ TextIconApplet.prototype = {
      * @panelHeight (int): height of the panel containing the applet
      * @instance_id (int): instance id of the applet
      */
-    _init: function(orientation, panel_height, instance_id) {
-        IconApplet.prototype._init.call(this, orientation, panel_height, instance_id);
+    _init(orientation, panel_height, instance_id) {
+        super._init(orientation, panel_height, instance_id);
         this._applet_label = new St.Label({ reactive: true,
                                             track_hover: true,
                                             style_class: 'applet-label'});
@@ -814,7 +862,7 @@ TextIconApplet.prototype = {
         this.actor.set_label_actor(this._applet_label);
 
         this.show_label_in_vertical_panels = true;
-    },
+    }
 
     /**
      * set_show_label_in_vertical_panels:
@@ -822,9 +870,9 @@ TextIconApplet.prototype = {
      *
      * Sets whether to show the label in vertical panels
      */
-    set_show_label_in_vertical_panels: function (show) {
+    set_show_label_in_vertical_panels (show) {
         this.show_label_in_vertical_panels = show;
-    },
+    }
 
     /**
      * set_applet_label:
@@ -832,7 +880,7 @@ TextIconApplet.prototype = {
      *
      * Sets the text of the actor to @text
      */
-    set_applet_label: function (text) {
+    set_applet_label (text) {
         this._applet_label.set_text(text);
 
         if ((this._orientation == St.Side.LEFT || this._orientation == St.Side.RIGHT) && (this.show_label_in_vertical_panels == false)) {
@@ -848,7 +896,7 @@ TextIconApplet.prototype = {
                 this.hide_applet_label(false);
             }
         }
-    },
+    }
 
     /**
      * set_applet_enabled:
@@ -857,7 +905,7 @@ TextIconApplet.prototype = {
      * Sets whether the applet is enabled or not. A disabled applet sets its
      * padding to 0px and doesn't react to clicks
      */
-    set_applet_enabled: function (enabled) {
+    set_applet_enabled (enabled) {
         if (enabled != this._applet_enabled) {
             this._applet_enabled = enabled;
             this.actor.visible = enabled;
@@ -865,7 +913,7 @@ TextIconApplet.prototype = {
                 this._applet_icon.visible = enabled;
             }
         }
-    },
+    }
 
     /**
      * hide_applet_label:
@@ -875,53 +923,41 @@ TextIconApplet.prototype = {
      * function to hide applet labels when an applet is placed in a vertical
      * panel
      */
-    hide_applet_label: function (hide) {
+    hide_applet_label (hide) {
         if (hide) {
             this.hideLabel();
         } else {
             this.showLabel();
         }
-    },
+    }
     /**
      * hideLabel:
      *
      * Hides the applet label
      */
-    hideLabel: function () {
+    hideLabel () {
         this._applet_label.hide();
         this._layoutBin.hide();
-    },
+    }
     /**
      * showLabel:
      *
      * Shows the applet label
      */
-    showLabel: function () {
+    showLabel () {
         this._applet_label.show();
         this._layoutBin.show();
-    },
+    }
     /**
      * hide_applet_icon:
      *
      * Hides the icon of the applet
      */
-    hide_applet_icon: function () {
+    hide_applet_icon () {
         this._applet_icon_box.child = null;
-    },
-
-    on_applet_added_to_panel: function() {
-
-    },
-
-    /**
-     * Override setOrientation, to recall set_applet_label
-     */
-    setOrientation: function (orientation) {
-        this.setOrientationInternal(orientation);
-        this.on_orientation_changed(orientation);
-        this.emit("orientation-changed", orientation);
-        this.finalizeContextMenu();
-        this._orientation = orientation;
-        this.set_applet_label(this._applet_label.get_text());
     }
-};
+
+    on_applet_added_to_panel() {
+
+    }
+}
